from .partials import PartialEmoji

class Reaction:
    def __init__(self, data: dict):
        self.count: int = data["count"]
        self.me: bool = data["me"]
        self.emoji: PartialEmoji = PartialEmoji(data["emoji"])


from typing import (
    Optional
)
class Attachment:
    def __init__(self, data: dict):
        self.id: str = data["id"]
        self.file_name: str = data["filename"]
        self.description: Optional[str] = data["description"] or None
        self.content_type: Optional[str] = data["content_type"] or None
        self.size: int = data["size"]
        self.proxy_url: str = data["proxy_url"]
        self.width: Optional[int] = data["width"] or None
        self.height: Optional[int] = data["height"] or None
        self.ephemeral: Optional[bool] = data["ephemeral"] or None


from .partials import PartialUser

from typing import (
    List,
    Optional
)

class TeamMember:
    def __init__(self, data: dict):
        self.data = data
        self.membership_state: int = data["membership_state"]
        self.team_id: str = data["team_id"]
        self.user: PartialUser = PartialUser(data["user"])

class Team:
    def __init__(self,data: dict):
        self.data = data
        self.icon: str = data["icon"]
        self.id: str = data["id"]
        self.members: List[TeamMember] = data["members"]


from typing import (
    Optional
)

from .client import ClientGuildMember, Client

class Guild:
    def __init__(self, client: Client, data: dict):
        self.member_cache: dict = {}
        self.client = client
        self.data: dict = data
        self.id: str = data["id"]
        self.name: str = data["name"]
        self.icon: Optional[str] = data["icon"] or None
        self.icon_hash: Optional[str] = data["icon_hash"] or None
        self.splash: Optional[str] = data["splash"] or None
        self.discovery_splash: Optional[str] = data["discovery_splash"] or None
        self.owner_id: str = data["owner_id"]
        self.me = ClientGuildMember(self.client)
        # self.me.permissions = data["permissions"]


from .abc import *
from .application import *
from .client import *
from .components import *
from .exceptions import *
from .guild import *
from .member import *
from .message import *
from .partials import *
from .role import *
from .route import *
from .team import *
from .user import *
from .ws import *

from .managers import *


from typing import (
    Optional,
    List
)

class PartialUser:
    def __init__(self, data: dict):
        self.data: dict = data
        self.id: str = data["id"]
        self.username: str = data["username"]
        self.discriminator: str = data["discriminator"]
        self.avatar: Optional[str] = data["avatar"]

class PartialEmoji:
    def __init__(self, data):
        self.data: dict = data
        self.name: str = data["name"]
        self.id: str = data["id"]
        self.animated: bool = data["animated"]
    
class PartialGuild:
    def __init__(self, data):
        self.data: dict = data
        self.id: str = data["id"]
        self.name: str = data["name"]
        self.permissions: int = int(data["permissions"])
        self.features: List[str] = data["features"]


from typing import (
    Optional
)
from .partials import PartialUser

class User(PartialUser):
    def __init__(self, data: dict):
        super().__init__(data)
        self.bot: bool = data["bot"]
        self.system: bool = data["system"]
        self.mfa_enabled: bool = data["mfa_enabled"]
        self.banner: Optional[str] = data["banner"] or None
        self.accent_color: Optional[int] = data["accent_color"] or None # the user's banner color encoded as an integer representation of hexadecimal color code	
        self.locale: Optional[str] = data["locale"] or None
        self.verified: bool = data["verified"]
        self.email: Optional[str] = data["email"] or None
        self.flags: int = data["flags"]
        self.premium_type: int = data["premium_type"]
        self.public_flags: int = data["public_flags"]


from typing import (
    List,
    Any
)
from .ws import WebsocketClient
from asyncio import run
from .member import Member
from .user import User
from .application import Application
from .route import Route
from aiohttp import ClientSession


class Client(WebsocketClient):

    def __init__(self, token: str, intents: int = 0, **options):
        super().__init__(token, intents)
        
        self.options: dict = options
        self.global_slash_commands: bool = options.get("global_slash_commands", True)
        self.slash_command_guilds: List[str] = options.get("slash_command_guilds", [])

        self.http = ClientSession(headers = {"Authorization": f"Bot {token}"}, base_url="https://discord.com/api/v9/")
        self.api = Route
        self.application: Application = Application(self, self.user) # Processes whatever it can        

class ClientUser(User):
    
    def __init__(self, client: Client, data: dict):
        super().__init__(data)
    
    async def fetch(self):
        response = await self.client.http.get("users/@me")
        data = await response.json()
        super().__init__(data) # Reinitialse the class with the new data.
    
class ClientGuildMember(Member):
    def __init__(self, client: Client,data: dict):
        super().__init__(data)


from distutils.core import setup
setup(
  name = 'EpikCord.py', 
  packages = ['EpikCord'], 
  version = '0.2',      
  license='MIT',        
  description = 'Have you ever wanted to make a Discord Bot? Are you upset that Discord.py has been discontinued? We introduce EpikCord.py, a Discord.py replacement.',
  long_description = "I present to you, EpikCord.py, this is a python package which will hopefully be a great replacement for Discord.py as the creator has decided to discontinue the package. Whilst there are forks of the discord.py repo, we have started this library from scratch. We hope you enjoy using it!",
  author = 'EpikHost',                   
  author_email = 'support@epikhost.xyz',      
  url = 'https://github.com/EpikHost/EpikCord.py',   
  #download_url = 'https://github.com/EpikHost/EpikCord.py/archive/v_01.tar.gz',
  keywords = ['EpikCord.py', 'Discord', 'API', "Bot"],   
  install_requires=[
          'aiohttp',
          "websocket-client"
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      
    'Intended Audience :: Developers',      
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',   
    'Programming Language :: Python :: 3.8',
  ],
)


from typing import (
    List
)
from .channels import TextBasedChannel
from .embed import Embed
from .client import Client
from .message import Message
from .exceptions import (
    InvalidArgumentType,
    CustomIdIsTooBig
)

class Messageable:
    def __init__(self, client: Client, channel_id: str):
        self.channel_id: str = channel_id
        self.client = client
        
    async def send(self, content: str, *, **kwargs) -> Message:
        
        response = await self.client.http.post(f"channels/{self.channel_id}/messages", data={"content": content})
        data = await response.json()
        return Message(data)
        
class BaseSlashCommandOption:
    def __init__(self, *, name: str, description: str, required: bool = False):
        self.name: str = name
        self.description: str = description
        self.required: bool = required
        # People shouldn't use this class, this is just a base class for other options

    def to_json(self):
        return {
            "name": self.name,
            "description": self.description,
            "required": self.required
        }
    
class BaseChannel:
    def __init__(self, data: dict):
        self.id: str = data["id"]
        self.type = data["type"]
        

class BaseComponent:
    def __init__(self):
        self.settings = {}

    def set_custom_id(self, custom_id: str):
        
        if not isinstance(custom_id, str):
            raise InvalidArgumentType("Custom Id must be a string.")
        
        elif len(custom_id) > 100:
            raise CustomIdIsTooBig("Custom Id must be 100 characters or less.")

        self.settings["custom_id"] = custom_id
        
class BaseInteraction:
    def __init__(self, data: dict):
        self.application_id: str = data["application_id"]
        self.channel: TextBasedChannel = 


from .thread import Thread
from .components import MessageSelectMenu, MessageButton
from .interactions import MessageInteraction
from .partials import PartialMember
from .role import Role
from .mentions import (
    MentionedUser,
    MentionedChannel
)    
from .application import Application
from .webhook import WebhookUser
from .user import User
from .reaction import Reaction
from typing import (
    Optional,
    List,
    Union
)
from .embed import Embed

class Message:
    def __init__(self, data: dict):
        self.id: str = data["id"]
        self.channel_id: str = data["channel_id"]
        self.guild_id: Optional[str] = data["guild_id"] or None
        self.webhook_id: Optional[str] = data["webhook_id"] or None
        self.author: Optional[User] if not self.webhook_id else WebhookUser = WebhookUser(data["author"]) if self.webhook_id else User(data["author"])
        self.member: Optional[PartialMember] = PartialMember(data["member"]) if data["member"] else None
        self.content: str = data["content"]
        self.timestamp: str = data["timestamp"]
        self.edited_timestamp: Optional[str] = data["edited_timestamp"] or None
        self.tts: bool = data["tts"]
        self.mention_everyone: bool = data["mention_everyone"]
        self.mentions: List[MentionedUser] = [MentionedUser(mention) for mention in data["mentions"]]
        self.mention_roles: List[Role] = [Role(role) for role in data["mention_roles"]]
        self.mention_channels: List[MentionedChannel] = [MentionedChannel(channel) for channel in data["mention_channels"]]
        self.embeds: List[Embed] = [Embed(embed) for embed in data["embeds"]]
        self.reactions: Optional[List[Reaction]] = [Reaction(reaction) for reaction in data["reactions"]]
        self.nonce: Optional[str] = data["nonce"] or None
        self.pinned: bool = data["pinned"]
        self.type: int = data["type"]
        self.activity: int = data["activity"]
        self.application: Application = Application(data["application"]) # Despite there being a PartialApplication, Discord don't specify what attributes it has
        self.flags: int = data["flags"]
        self.referenced_message: Optional[Message] = Message(data["referenced_message"]) if data["referenced_message"] else None
        self.interaction: Optional[MessageInteraction] = MessageInteraction(data["interaction"]) if data["interaction"] else None
        self.thread: Thread = Thread(data["thread"]) if data["thread"] else None
        self.components: List[Union[MessageSelectMenu, MessageButton]]